@@ -3,6 +3,7 @@
 """API Extension."""
 
 from flask import Blueprint, jsonify, request
+from flask.views import View
 from os.path import abspath, dirname, join
 from sqlalchemy import Column
 from sqlalchemy.ext.declarative import declarative_base, declared_attr 
@@ -15,6 +16,158 @@ from werkzeug.exceptions import HTTPException
 from ..project import current_project as pj
 from ..util import Cacheable, JSONEncodedDict, _jsonify, Loggable, uncamelcase
 
+class API(object):
+
+  """Main API extension.
+
+  Handles the creation and registration of all API views. Available options:
+
+  * URL_PREFIX
+  * ADD_ALL_MODELS
+  * DEFAULT_METHODS
+  * DEFAULT_ALLOW_PUT_MANY
+  * DEFAULT_RELATIONSHIPS
+  * DEFAULT_COLLECTION_DEPTH
+  * DEFAULT_MODEL_DEPTH
+  * DEFAULT_LIMIT
+  * MAX_LIMIT
+
+  Models can be added in two ways. Either individually::
+    
+    api.add_model(Model)
+
+  or globally::
+
+    api.add_all_models()
+
+  Both functions accept the same additional options (cf. their respective doc).
+
+  It also exposes the `authorize` and `validate` decorators.
+
+  Once all the models have been added along with (optionally) the authorize and
+  validate functions, the API extension should be registered with the project::
+
+    current_project.register_extension(extension)
+
+  """
+
+  _authorize = None
+  _validate = None
+
+  config = {
+    'URL_PREFIX': '/api',
+    'ADD_ALL_MODELS': False,
+    'METHODS': frozenset(['GET', 'POST', 'PUT', 'DELETE']),
+    'RELATIONSHIPS': True,
+    'DEFAULT_DEPTH': 0,
+    'DEFAULT_LIMIT': 20,
+    'MAX_LIMIT': None,
+  }
+
+  def __init__(self, **kwargs):
+    for k, v in kwargs.items():
+      self.config[k.upper()] = v
+    self.Models = {}
+    APIView.extension = self
+
+  def add_model(self, Model, **kwargs):
+    """Flag a Model to be added.
+
+    TODO: kwargs (relationships, methods...)
+    
+    Will override any options previously set for that Model.
+
+    If any of the options is ``None``, the default value will be used.
+
+    Relationships can either be ``True`` or a list of relationship keys. In the
+    first case, all one to many relationships will have a hook created,
+    otherwise only those mentioned in the list.
+
+    It might seem strange that this method doesn't have a columns filter, this
+    is for speed and consistency. Columns are defined at the Model level (via
+    the json_include and json_exclude attributes), this way:
+    
+    * jsonify calls do not have to dynamically check which columns to include
+    * all endpoints leading to the same Model yield the same columns
+
+    """
+    self.Models[Model.__name__] = (Model, kwargs)
+
+  def authorize(self, func):
+    """Decorator to set the authorizer function.
+
+    The authorizer is passed three arguments:
+    
+    * the model class
+    * the relationship (or ``None``)
+    * the request method 
+
+    The third argument is for convenience (the full request object can be 
+    accessed as usual).
+
+    If the function returns a truthful value, the request will proceed.
+    Otherwise a 403 exception is raised.
+    
+    """
+    self._authorize = func
+
+  def validate(self, func):
+    """Decorator to set the validate function, called on POST and PUT.
+
+    The validator will be passed two arguments:
+
+    * the model class
+    * the request json
+    * the request method
+
+    If the function returns a truthful value, the request will proceed.
+    Otherwise a 400 exception is raised.
+
+    """
+    self._validate = func
+
+  def _create_model_views(self, Model, options=None):
+    """Creates the views associated with the model.
+
+    Sane defaults are chosen:
+
+    * PUT and DELETE requests are only allowed for endpoints that correspond
+      to a single model
+    * POST requests are only allowed for endpoints that correspond to
+      collections
+    * endpoints corresponding to relationships only allow the GET method
+      (this choice was made to avoid duplicating features accross endpoints)
+
+    """
+    CollectionView.attach_view(Model, None)
+    ModelView.attach_view(Model, None)
+    for rel in Model.get_relationships():
+      if rel.uselist:
+        CollectionView.attach_view(Model, rel)
+        ModelView.attach_view(Model, rel)
+
+  def _before_register(self, project):
+    self.blueprint = Blueprint(
+      'api',
+      project.config['PROJECT']['APP_FOLDER'] + '.api',
+      template_folder=abspath(join(dirname(__file__), 'templates', 'api')),
+      url_prefix=self.config['URL_PREFIX']
+    )
+    if self.config['ADD_ALL_MODELS']:
+      Models = [k.class_ for k in mapperlib._mapper_registry]
+      for Model in Models:
+        if not Model.__name__ in self.Models:
+          self.add_model(Model)
+    for Model, options in self.Models.values():
+      self._create_model_views(Model, options)
+    IndexView.attach_view(None, None)
+
+  def _after_register(self, project):
+    Model.metadata.create_all(project._engine, checkfirst=True)
+    Model.query = _QueryProperty(project)
+
+# SQLAlchemy Model
+
 class _BaseQuery(Query):
 
   """Base query class.
@@ -191,6 +344,8 @@ class ExpandedBase(Cacheable, Loggable):
 
 Model = declarative_base(cls=ExpandedBase)
 
+# Views
+
 class APIError(HTTPException):
 
   """Thrown when an API call is invalid."""
@@ -203,217 +358,40 @@ class APIError(HTTPException):
   def __repr__(self):
     return '<APIError %r: %r>' % (self.message, self.content)
 
-class API(object):
-
-  """Main API extension.
-
-  Handles the creation and registration of all API views. Available options:
-
-  * URL_PREFIX
-  * ADD_ALL_MODELS
-  * DEFAULT_METHODS
-  * DEFAULT_ALLOW_PUT_MANY
-  * DEFAULT_RELATIONSHIPS
-  * DEFAULT_COLLECTION_DEPTH
-  * DEFAULT_MODEL_DEPTH
-  * DEFAULT_LIMIT
-  * MAX_LIMIT
-
-  Models can be added in two ways. Either individually::
-    
-    api.add_model(Model)
-
-  or globally::
-
-    api.add_all_models()
-
-  Both functions accept the same additional options (cf. their respective doc).
-
-  It also exposes the `authorize` and `validate` decorators.
-
-  Once all the models have been added along with (optionally) the authorize and
-  validate functions, the API extension should be registered with the project::
-
-    current_project.register_extension(extension)
-
-  """
-
-  _authorize = None
-  _validate = None
-
-  config = {
-    'URL_PREFIX': '/api',
-    'ADD_ALL_MODELS': False,
-    'METHODS': frozenset(['GET', 'POST', 'PUT', 'DELETE']),
-    'ALLOW_PUT_MANY': False,
-    'RELATIONSHIPS': True,
-    'DEFAULT_COLLECTION_DEPTH': 0,
-    'DEFAULT_MODEL_DEPTH': 1,
-    'DEFAULT_LIMIT': 20,
-    'MAX_LIMIT': None,
-  }
-
-  def __init__(self, **kwargs):
-    for k, v in kwargs.items():
-      self.config[k.upper()] = v
-    self.Models = {}
-    APIView._extension = self
-
-  def add_model(self, Model, relationships=None, allow_put_many=None,
-                methods=None):
-    """Flag a Model to be added.
-    
-    Will override any options previously set for that Model.
-
-    If any of the options is ``None``, the default value will be used.
-
-    Relationships can either be ``True`` or a list of relationship keys. In the
-    first case, all one to many relationships will have a hook created,
-    otherwise only those mentioned in the list.
-
-    It might seem strange that this method doesn't have a columns filter, this
-    is for speed and consistency. Columns are defined at the Model level (via
-    the json_include and json_exclude attributes), this way:
-    
-    * jsonify calls do not have to dynamically check which columns to include
-    * all endpoints leading to the same Model yield the same columns
-
-    """
-    self.Models[Model.__name__] = (
-      Model, methods, relationships, allow_put_many
-    )
-
-  def authorize(self, func):
-    """Decorator to set the authorizer function.
-
-    The authorizer is passed three arguments:
-    
-    * the model class
-    * the relationship (or ``None``)
-    * the request method 
-
-    The third argument is for convenience (the full request object can be 
-    accessed as usual).
-
-    If the function returns a truthful value, the request will proceed.
-    Otherwise a 403 exception is raised.
-    
-    """
-    self._authorize = func
-
-  def validate(self, func):
-    """Decorator to set the validate function, called on POST and PUT.
-
-    The validator will be passed two arguments:
-
-    * the model class
-    * the request json
-    * the request method
-
-    If the function returns a truthful value, the request will proceed.
-    Otherwise a 400 exception is raised.
-
-    """
-    self._validate = func
-
-  def _create_model_views(self, Model, relationships, allow_put_many, methods):
-    """Creates the views associated with the model.
-
-    Sane defaults are chosen:
-
-    * PUT and DELETE requests are only allowed for endpoints that correspond
-      to a single model
-    * POST requests are only allowed for endpoints that correspond to
-      collections
-    * endpoints corresponding to relationships only allow the GET method
-      (this choice was made to avoid duplicating features accross endpoints)
-
-    """
-    views = []
-    methods = set(methods)
-    model_methods = methods & set(['GET', 'PUT', 'DELETE'])
-    if model_methods:
-      views.append(ModelView(Model=Model, methods=model_methods))
-    collection_methods = methods & set(['GET', 'POST'])
-    if collection_methods:
-      views.append(CollectionView(Model=Model, methods=collection_methods))
-    if allow_put_many:
-      col_methods = set(['GET', 'PUT']) & methods
-    else:
-      col_methods = set(['GET']) & methods
-    rel_methods = set(['GET']) & methods
-    if rel_methods:
-      for rel in Model.get_relationships():
-        if rel.key in relationships and rel.uselist:
-          views.extend([
-            ModelView(Model=Model, relationship=rel, methods=rel_methods),
-            CollectionView(Model=Model, relationship=rel, methods=col_methods)
-          ])
-    for view in views:
-      self.blueprint.add_url_rule(view.url, view.endpoint, view)
-
-  def _before_register(self, project):
-    self.blueprint = Blueprint(
-      'api',
-      project.config['PROJECT']['APP_FOLDER'] + '.api',
-      template_folder=abspath(join(dirname(__file__), 'templates', 'api')),
-      url_prefix=self.config['URL_PREFIX']
-    )
-    if self.config['ADD_ALL_MODELS']:
-      Models = [k.class_ for k in mapperlib._mapper_registry]
-      for Model in Models:
-        if not Model.__name__ in self.Models:
-          self.add_model(Model)
-    for Model, methods, rels, batch in self.Models.values():
-      if methods is None: methods = self.config['METHODS']
-      if batch is None: batch = self.config['ALLOW_PUT_MANY']
-      if rels is None: rels = self.config['RELATIONSHIPS']
-      if rels is True:
-        rels = set([r.key for r in Model.get_relationships()])
-      elif rels is False:
-        rels = set()
-      self.Models[Model.__name__] = (Model, methods, rels, batch)
-      self._create_model_views(Model, rels, batch, methods)
-    index_view = IndexView()
-    self.blueprint.add_url_rule(
-      index_view.url, index_view.endpoint, index_view
-    )
-
-  def _after_register(self, project):
-    Model.metadata.create_all(project._engine, checkfirst=True)
-    Model.query = _QueryProperty(project)
-
-class APIView(object):
+class APIView(View):
 
   """Base API view.
 
   Note that the methods class attribute seems to be passed to the add_url_rule
   function somehow (!).
-  
+
+  A collection can either be:
+
+  * a query (most cases)
+  * an instrumented list (in the case of relationships)
+
   """
 
-  __all__ = []
 
-  _extension = None
+  # Flask stuff
+  decorators = []
+  methods = frozenset(['get', 'post', 'head', 'options',
+                       'delete', 'put', 'trace', 'patch'])
 
-  Model = None
-  relationship = None
-  methods = set()
-  params = set()
+  extension = None
+  routes = []
 
-  def __init__(self, **kwargs):
-    for k, v in kwargs.items():
-      setattr(self, k, v)
-    self.__all__.append(self)
+  def __init__(self, Model, rel):
+    self.Model = Model
+    self.rel = rel
 
-  def __call__(self, **kwargs):
-    """Redirects to corresponding request handler and catch APIErrors."""
+  def dispatch_request(self, *args, **kwargs):
+    method = getattr(self, request.method.lower(), None)
+    if method is None and request.method == 'HEAD':
+      method = getattr(self, 'get', None)
     try:
-      if not self.is_authorized():
-        raise APIError(403, 'Not authorized')
-      if request.method in self.methods:
-        params = self._parse_params()
-        return getattr(self, request.method.lower())(params, **kwargs)
+      if method:
+        return method(*args, **kwargs)
       else:
         raise APIError(405, 'Method Not Allowed')
     except APIError as e:
@@ -427,115 +405,50 @@ class APIView(object):
         'content': e.content
       }), e.code
 
-  @property
-  def available_columns(self):
-    return self.Model._json_attributes
+  @classmethod
+  def attach_view(cls, Model, rel):
+    route = cls.get_url(Model, rel)
+    cls.extension.blueprint.add_url_rule(
+      route,
+      view_func=cls.as_view(cls.get_endpoint(Model, rel), Model)
+    )
+    cls.routes.append(route)
 
-  @property
-  def available_relationships(self):
-    return [
-      r for r in self.Model.get_relationships()
-      if r.uselist
-      if r.key in self._extension.Models[self.Model.__name__][2]
-    ]
-
-  @property
-  def authorized_methods(self):
-    return set(
-      method for method in self.methods 
-      if self.is_authorized(method)
+  @classmethod
+  def get_endpoint(cls, Model, relationship):
+    return '%s%s%s' % (
+      uncamelcase(cls.__name__),
+      '_for_%s' % Model.__tablename__ if Model else '',
+      '.%s' % relationship.key if relationship else '',
     )
 
-  @property
-  def defaults(self):
-    return self._extension.config
-
-  def is_validated(self, json):
-    """Validate changes."""
-    if not self._extension._validate:
-      return True
-    return self._mananager._validate(self.Model, json, request.method)
-
-  def is_authorized(self, method=None):
-    if method is None:
-      method = request.method
-    if not self._extension._authorize:
-      return True
-    return self._extension._authorize(self.Model, self.relationship, method)
-
-  def _parse_params(self):
-    return request.args
+  @classmethod
+  def get_url(cls, Model, relationship):
+    url = getattr(cls, 'url', None)
+    if not url:
+      raise NotImplementedError
+    return url
 
 class IndexView(APIView):
 
   """API 'splash' page with a few helpful keys."""
 
-  methods = set(['GET'])
+  url = '/'
 
-  @property
-  def endpoint(self):
-    return 'index'
- 
-  @property
-  def url(self):
-    return '/'
-
-  def get(self, params, **kwargs):
+  def get(self, **kwargs):
     return jsonify({
       'status': '200 Welcome',
-      'available_endpoints': [
-        '%s (%s)' % (v.url, ', '.join(v.authorized_methods))
-        for v in self.__all__
-        if v.authorized_methods
-      ],
-      'available_models': dict(
-        (
-          v.Model.__name__,
-          {
-            'available_columns': v.available_columns,
-            'available_relationships': [
-              r.key for r in v.available_relationships
-            ]
-          }
-        ) for v in self.__all__
-        if v.Model
-        if not v.relationship
-      )
+      'available_endpoints': self.routes
     })
 
 class CollectionView(APIView):
 
   """View for collection endpoints."""
 
-  params = set(['depth', 'limit', 'offset', 'loaded', 'sort'])
-
-  @property
-  def endpoint(self):
-    if not self.relationship:
-      return '%s_collection_view' % self.Model.__tablename__
-    else:
-      return '%s_%s_collection_view' % (
-        self.Model.__tablename__, self.relationship.key
-      )
-
-  @property
-  def url(self):
-    url = '/%s' % self.Model.__tablename__
-    if self.relationship:
-      url += ''.join('/<%s>' % n for n in self.Model.get_primary_key_names())
-      url += '/%s' % self.relationship.key
-    url += '/'
-    return url
-
-  def get(self, params, **kwargs):
-    if not kwargs:
-      query = self.Model.query
-    else:
-      model = self.Model.query.get(kwargs.values())
-      if not model:
-        raise APIError(404, 'No resource found for this ID')
-      query = getattr(model, self.relationship.key)
-    if isinstance(query, InstrumentedList):
+  def get(self, **kwargs):
+    params = self.get_params()
+    col = self._get_collection(**kwargs)
+    if isinstance(col, InstrumentedList):
       results = self._process_list(query, params)
     else:
       results = self._process_query(query, params)
@@ -554,7 +467,7 @@ class CollectionView(APIView):
       'content': results['content']
     }), 200
 
-  def post(self, params, **kwargs):
+  def post(self, **kwargs):
     if self.is_validated(request.json):
       model = Model(**request.json)
       pj.session.add(model)
@@ -577,6 +490,17 @@ class CollectionView(APIView):
     else:
       raise APIError(404, 'No resource found for this ID')
 
+  def _get_collection(self, **kwargs):
+    """Getting raw query/list."""
+    if not kwargs:
+      col = self.Model.query
+    else:
+      model = self.Model.query.get(kwargs.values())
+      if not model:
+        raise APIError(404, 'No resource found for this ID')
+      col = getattr(model, self.rel.key)
+    return col
+
   def _process_list(self, lst, params):
     return {
       'processing_times': [],
@@ -637,27 +561,25 @@ class CollectionView(APIView):
       ]
     }
 
+  @classmethod
+  def get_url(self, Model, relationship):
+    url = '/%s' % Model.__tablename__
+    if relationship:
+      url += ''.join('/<%s>' % n for n in Model.get_primary_key_names())
+      url += '/%s' % relationship.key
+    url += '/'
+    return url
+
 class ModelView(APIView):
 
   """View for individual model endpoints."""
 
-  params = set(['depth'])
-
-  @property
-  def endpoint(self):
-    if not self.relationship:
-      return '%s_model_view' % self.Model.__tablename__
-    else:
-      return '%s_%s_model_view' % (
-        self.Model.__tablename__, self.relationship.key
-      )
-
-  @property
-  def url(self):
-    url = '/%s' % self.Model.__tablename__
-    url += ''.join('/<%s>' % n for n in self.Model.get_primary_key_names())
-    if self.relationship:
-      url += '/%s/<position>' % self.relationship.key
+  @classmethod
+  def get_url(self, Model, relationship):
+    url = '/%s' % Model.__tablename__
+    url += ''.join('/<%s>' % n for n in Model.get_primary_key_names())
+    if relationship:
+      url += '/%s/<position>' % relationship.key
     return url
 
   def get(self, params, **kwargs):
@@ -705,3 +627,35 @@ class ModelView(APIView):
     else:
       raise APIError(404, 'No resource found for this ID')
 
+# Helper
+
+class RequestParser(object):
+
+  params = frozenset(['filter', 'depth', 'limit', 'sort'])
+  sep = ';'
+
+  def __init__(self, allowed_params):
+    self.args = request.args
+    allowed_params = allowed_params or self.params
+    if not set(self.args.keys()) < allowed_params:
+      raise APIError(400, 'Invalid parameter found')
+
+  def depth(self):
+    return request.args.get('depth', 0, int)
+
+  def offset(self):
+    return self.args.get('offset', 0, int)
+
+  def limit(self):
+    return self.args.get('limit', 20, int)
+
+  def filters(self):
+    raws = self.args.getlist('filter')
+    for raw in raws:
+      key, op, value = raw.split(self.sep, 3)
+
+  def sorts(self):
+    raws = self.args.getlist('sort')
+    for raw in raws:
+      key, order = raw.split(self.sep)
+
